#ifndef PMAINWINDOW_H_
#define PMAINWINDOW_H_
/*
 *   QPcr1k 
 *   Copyright (C) 1999 by PolyWog and Javaman for Ghetto.Org
 *    <polywog@ghetto.org>, <javaman@ghetto.org>
 *
 *   A GUI front end using the QT Toolkit and PCP/PCOMM library
 *   for the Icom Pcr-1000 radio.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <qapp.h>
#include <qmainwindow.h>

#include "pfileio.h"

#ifdef DEBUG_VER_
#include <stdio.h>
#endif // DEBUG_VER_

#define QPCR1K_VERSION "QP_V1000\n"

class QTimer;
class PCP;
class PUI;
class PMenu;
class PTools;
class PStatus;
class PFileIO;

class PMainWindow : public QMainWindow
{
	Q_OBJECT

	public:
	 PMainWindow(QWidget *parent=0, const char *name=0);
	 ~PMainWindow();

	private slots:
   	 void execRadioCommandSlot(const char *, const char*);
	 void updateStatusSlot();

 	 void PMAbout();
 	 void PMNewProfile();
 	 void PMLoadProfile(); 
	 void PMSaveAsProfile(); 
	 void PMSaveProfile(); 
	 void PMQuit(); 
	 void PMEditProfile(); 
	 void PMInit(); 
	 void PMPower();

	private:
	 bool   execRadioCommand(const char*, const char*);
	
	 PFileIO     *PMRcFile;
	 qpcrRc       PMRc;
	 bool         PMStatus;      /* radio on/off (true/false) */ 
	 PCP         *PMRadio;       /* PCR1k Communication Device */ 
	 // we should have an object here that holds radio data 
	 PMenu       *PMMenuBar;     /* Main Menu Bar */ 
	 PTools      *PMToolBar;     /* Main Tool Bar */ 
	 PUI         *PMInterface;   /* User Interface Widget */ 
	 PStatus     *PMStatusBar;   /* Main Status Bar */ 

	 QTimer	     *PMStatTimer;   /* Status Bar Timer */
};

#endif /* PMAINWINDOW_H_ */
