/*
 *   QPcr1k
 *   Copyright (C) 1999 by PolyWog and Javaman for Ghetto.Org
 *    <polywog@ghetto.org>, <javaman@ghetto.org>
 *
 *   A GUI front end using the QT Toolkit and PCP/PCOMM library
 *   for the Icom Pcr-1000 radio.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#if defined(SunOS) || defined(Irix)
#include <strings.h>
#else
#include <string.h>
#endif


#include <qlabel.h>
#include <qlineedit.h>
#include <qgroupbox.h>
#include <qsplitter.h>
#include <qlistview.h>
#include <qlayout.h>
#include <qpushbutton.h>

#include <stdlib.h> // for free() against the malloc()

#include "prodlg.h"
#include "pfileio.h"

ProDlg :: ProDlg (qpcrRc *passedRc, QWidget *parent, const char *name, bool modalness)
         : QDialog(parent, name, modalness)
{
    #ifdef DEBUG_VER_
    fprintf(stderr, "ProDlg: Creating...\n");
    fflush(stderr);
    #endif // DEBUG_VER_

    tempRc = new qpcrRc;
    bzero(tempRc, sizeof(tempRc));
    memcpy(tempRc, passedRc, sizeof(qpcrRc));

    // create a layout object (in this case a grid layout 1x1) so
    // that our widget sizes can be resized automagically. Then
    // create a splitter, and add the splitter to the grid layout.
    PDlMainGL   = new QGridLayout( this, 1, 1, 1, 1, "pdlmaingl");
    PDlMainSplit = new QSplitter( QSplitter::Horizontal, this );
    PDlMainGL->addWidget( PDlMainSplit, 0, 0 );

    // create the listview to left of the splitter, add a column
    // and set some pretty options, then connect the listview's
    // signal (selectionChanged(QListViewItem *)) to _this_'s
    // slot (PDlSelectionChangedSlot(QListViewItem *))
    PDlOptsLV   = new QListView( PDlMainSplit );
    PDlOptsLV->addColumn("Main Options");
    PDlOptsLV->addColumn("Frequency");
    PDlOptsLV->addColumn("Mode");
    PDlOptsLV->addColumn("Filter");
    PDlOptsLV->addColumn("Tone Sq");
    PDlOptsLV->addColumn("Description");

    PDlOptsLV->setRootIsDecorated( TRUE );
    connect (PDlOptsLV, SIGNAL(selectionChanged( QListViewItem * )),
             this,      SLOT(PDlSelectionChangedSlot( QListViewItem * )) );

    // add the two parent listview items. There may be more later.
    QString channels("Channels");
    QString globals("Globals");
    PDlGlobalsLVI[0] = new QListViewItem(PDlOptsLV, globals);
    PDlChannelsLVI[0] = new QListViewItem(PDlOptsLV, channels);

    // automagically populated
        int i=0;
        QString strRcMem;
        while ((i<=tempRc->RcMaxCh)&&(i<40)) {
            strRcMem.setNum(i);
            PDlChannelsLVI[1] = new QListViewItem (PDlChannelsLVI[i],
                    strRcMem,
                    tempRc->RcMem[i].ChFreq,
                    tempRc->RcMem[i].ChMode,
                    tempRc->RcMem[i].ChFilter,
                    tempRc->RcMem[i].ChToneSq,
                    tempRc->RcMem[i].ChDesc);
            i++;
        }

        QString globals1("Device");
        QString globals2("BaudRate");
        PDlGlobalsLVI[1] = new QListViewItem (PDlGlobalsLVI[0], globals1);
        PDlGlobalsLVI[2] = new QListViewItem (PDlGlobalsLVI[0], globals2);
    // by a loop that will read the struct

    PDlOptsLV->setMinimumWidth( PDlOptsLV->sizeHint().width() );

    // make a group box that will gather our editing abilities
    // together. Make its parent the splitter so it will appear
    // on the right of the splitter. Give the group box one column
    // and set the orientation to horizontal.
    PDlEditGB = new QGroupBox("Info", PDlMainSplit);
    PDlEditGB->setColumnLayout(1, QGroupBox::Horizontal);

    // set some default stuff. While making its parent the group box
    PDlNameLabel = new QLabel("Drag the slider on\n"
                              "the left to access\n"
                              "multiple options",   PDlEditGB);
    PDlValueLE   = new QLineEdit(PDlEditGB);
    PDlValueLE->hide();

    PDlCommitPB     = new QPushButton("Commit", PDlEditGB );
    connect(PDlCommitPB, SIGNAL(clicked()), SLOT(PDlCommitPBSlot()));
    PDlCommitPB->hide();
    PDlRollbackPB   = new QPushButton("Rollback", PDlEditGB );
    connect(PDlRollbackPB, SIGNAL(clicked()), SLOT(PDlRollbackPBSlot()));
    PDlRollbackPB->hide();

    // activate grid layout where all of this will live
    PDlMainGL->activate();

    #ifdef DEBUG_VER_
    fprintf(stderr, "ProDlg: Created!\n");
    #endif // DEBUG_VER_
}

ProDlg :: ~ProDlg()
{
    #ifdef DEBUG_VER_
    fprintf(stderr, "ProDlg: Destroyed!\n");
    #endif // DEBUG_VER_

    free(tempRc);
}

void ProDlg :: PDlSelectionChangedSlot( QListViewItem *currItem )
/*
 *  This slot is connected to the listview. Basically, the way it
 *  works ir rather simple. If there are any name/value/info boxes
 *  in the edit group box in the right pane, delete them.
 *
 *  If we were fed a QListViewItem then set the groupbox title to
 *  the listitem's main name (the name in col index 0). Then create
 *  the appropriate name/value pairs.
 *
 *  If we werent fed a QListViewItem then set it to the default
 *  description box.
 */
{
#ifdef DEBUG_VER_
    fprintf( stderr, "PRODLG: PDLSELECTIONCHANGEDSLOT\n");
    fflush( stderr );
#endif // DEBUG_VER_

/*    if ( PDlInfoLabel!=NULL ) { delete PDlInfoLabel; PDlInfoLabel=0; }
    if ( PDlNameLabel!=NULL ) { delete PDlNameLabel; PDlNameLabel=0; }
    if ( PDlValueLE  !=NULL ) { delete PDlValueLE;   PDlValueLE=0;   }
    if ( PDlCommitPB !=NULL ) { delete PDlCommitPB;  PDlCommitPB=0;  }
    if ( PDlRollbackPB!=NULL) { delete PDlRollbackPB;PDlRollbackPB=0;}
    if ( PDlEditGB   !=NULL ) { delete PDlEditGB;    PDlEditGB=0;    } */

    if (currItem!=0) {
        PDlEditGB->setTitle(currItem->text(0));
        if (strcmp(currItem->text(0),"Device")==0) {
            PDlNameLabel->setText("Device");
            PDlValueLE->setText("device-name-goes-here");
            PDlValueLE->show();
            PDlCommitPB->show();
            PDlRollbackPB->show();
        } else if (strcmp(currItem->text(0),"BaudRate")==0) {
            PDlNameLabel->setText("BaudRate");
            PDlValueLE->setText("B38400");
            PDlValueLE->show();
            PDlCommitPB->show();
            PDlRollbackPB->show();
        } else if (strcmp(currItem->text(0),"Globals")==0) {
            PDlNameLabel->setText("Global Settings");
            PDlValueLE->hide();
            PDlCommitPB->hide();
            PDlRollbackPB->hide();
        } else if (strcmp(currItem->text(0),"Channels")==0) {
            PDlNameLabel->setText("Memory Channels");
            PDlValueLE->hide();
            PDlCommitPB->hide();
            PDlRollbackPB->hide();
        } else {
            PDlEditGB->setTitle("Info");
            PDlNameLabel->setText("Drag the slider on\n"
                              "the left to access\n"
                              "multiple options");
            PDlValueLE->hide();
            PDlCommitPB->hide();
            PDlRollbackPB->hide();
        }
    }

}

void ProDlg :: PDlCommitPBSlot()
{
    accept();
}

void ProDlg :: PDlRollbackPBSlot()
{
    reject();
}
